#a tree form widget that
#!/usr/bin/python
# -*- coding: utf-8 -*-
from django import forms
from django.forms import fields
from django.utils.safestring import mark_safe
from django.template.loader import get_template
from django.template.context import Context
from django.forms.widgets import Widget, Select
from simple_locations.models import Area,AreaType
from django.conf import settings
from mptt.forms import TreeNodeChoiceField
from django.forms.utils import ErrorList
from code_generator.code_generator import generate_tracking_tag

class LocationForm(forms.Form):
    name = forms.CharField(max_length=100)
    code = forms.CharField(max_length=50,required=False)
    pk = forms.CharField(widget=forms.HiddenInput(), required=False)
    target = TreeNodeChoiceField(queryset=Area.tree.all(),
                                 level_indicator=u'++', required=False)
    lat = forms.DecimalField(required=False)
    lon = forms.DecimalField(required=False)
    kind=forms.ChoiceField(required=False,
                           choices=(('','-----'),)+tuple([(int(w.pk),w.name) for w in AreaType.objects.all() ]))
    move_choice = forms.BooleanField(required=False)
    position = forms.ChoiceField(choices=(('last-child', 'inside'), ('left', 'before'), ('right', 'after')),
                                 required=False)
    def clean(self):
        """ make sure that both lat and lon are provided. if lat is given then lon is also required and vice versa.  """
        lat=self.cleaned_data['lat']
        lon=self.cleaned_data['lon']
        code=self.cleaned_data['code']
        if not lat and lon:
            msg=u"Please provide the latitude"
            self._errors["lat"]=ErrorList([msg])
            return ''
        elif lat and not lon:
            msg=u"Please provide the longitude"
            self._errors["lon"]=ErrorList([msg])
            return ''
        if lat and lon:
            if not -90 <= lat <= 90 :
                msg=u'Invalid latitude must be between 90 and -90'
                self._errors["lat"]=ErrorList([msg])
                return ''
            if not -180 <= lon <=180 :
                msg=u'Invalid latitude must be between 180 and -180'
                self._errors["lon"]=ErrorList([msg])
                return ''
        if not code:
            try:
                self.cleaned_data['code']=generate_tracking_tag(Area.objects.order_by('-pk')[0].code)
            except:
                self.cleaned_data['code']=generate_tracking_tag()

        else:
            self.cleaned_data['code']='2'+code

        return self.cleaned_data
    
    
   
    
    
            
            
    





from django.conf.urls import url
from simple_locations.views import edit_location, add_location, delete_location, render_location, simple_locations, area_search

urlpatterns = [
    url(r'^simple_locations/edit/(?P<area_id>[0-9]+)/$', edit_location),
    url(r'^simple_locations/add/((?P<parent_id>[0-9]+)/){0,1}$', add_location),
    url(r'^simple_locations/delete/(?P<area_id>[0-9]+)/$', delete_location),
    url(r'^simple_locations/render_tree/$', render_location),
    url(r'^simple_locations/$', simple_locations),
    url(r'^areasearch/$', area_search),
]



from django.core.management.base import BaseCommand
from simple_locations.models import Area

class Command(BaseCommand):

    help = """prints all districts in a form that's easy to import into excel"""

    def handle(self, **options):
        print('id\tname\ttype\tparent_id\tparent_name\tparent_type')
        for a in Area.tree.all():
            print("%d\t%s\t%s\t%d\t%s\t%s" % (a.pk, a.name, a.kind.name, a.parent.pk if a.parent else -1, a.parent.name if a.parent else '', a.parent.kind.name if a.parent else ''))



#!/usr/bin/python
# -*- coding: utf-8 -*-
from django.shortcuts import render_to_response, get_object_or_404
from django.template import RequestContext
from simple_locations.models import Area,Point,AreaType
from django.db import IntegrityError
from forms import LocationForm
from django.http import HttpResponseRedirect
from mptt.exceptions import InvalidMove
from django.views.decorators.cache import cache_control
from django.conf import settings
from django.core import serializers
from django.http import HttpResponse

#firefox likes to aaggressively cache forms set cache control to false to override this
@cache_control(no_cache=True)
def simple_locations(request):
    form = LocationForm()
    nodes = Area.tree.all()
    return render_to_response(
            'simple_locations/index.html',
            {'form' : form,
             'nodes':nodes,
             'map_key':settings.MAP_KEY,
            },
            context_instance = RequestContext(request))

def add_location(req, parent_id=None):
    nodes = Area.tree.all()

    if req.method == 'POST':
        form = LocationForm(req.POST)
        if form.is_valid():
            name = form.cleaned_data['name']
            code = form.cleaned_data['code']
            lat = form.cleaned_data['lat']
            lon = form.cleaned_data['lon']
            target = form.cleaned_data['target']
            position = form.cleaned_data['position']
            kind=form.cleaned_data['kind']

            area=Area.objects.create(name=name,code=code,parent=target)
            if lat and lon:
                location=Point(latitude=lat,longitude=lon)
                location.save()
                area.location=location
            try:
                kind=get_object_or_404(AreaType,pk=int(kind))
                area.kind=kind
            except ValueError:
                pass
            area.save()
            form = LocationForm()

            return render_to_response(
                    'simple_locations/location_edit.html'
                    ,{'form': form, 'nodes': nodes},
                    context_instance=RequestContext(req))
        else:
            form = LocationForm(req.POST)
            return render_to_response(
                    'simple_locations/location_edit.html'
                    ,{'form': form, 'nodes': nodes},
                    context_instance=RequestContext(req))

    else:
        if (parent_id):
            default_data = {}
            parent = get_object_or_404(Area, pk=parent_id)
            default_data['move_choice'] = True
            default_data['target'] = parent.pk
            default_data['position'] = 'last-child'
            form = LocationForm(default_data)
            form._errors=''


        else:
            form = LocationForm()

    return render_to_response(
            'simple_locations/location_edit.html'
            ,{'form': form, 'nodes': nodes},
            context_instance=RequestContext(req))

def edit_location(req, area_id):
    location = get_object_or_404(Area, pk=area_id)
    if req.method == 'POST':
        form = LocationForm(req.POST)
        if form.is_valid():
            saved = True

            area = Area.objects.get(pk=area_id)
            area.name = form.cleaned_data['name']
            area.code = form.cleaned_data['code']
            lat=form.cleaned_data['lat']
            lon=form.cleaned_data['lon']
            kind=form.cleaned_data['kind']
            try:
                kind = get_object_or_404(AreaType,pk=int(kind))
                area.kind = kind
            except ValueError:
                pass
            if lat and lon:
                point = Point(latitude=lat,longitude=lon)
                point.save()
                area.location = point
            try:
                area.save()
            except IntegrityError:
                form.errors['code'] = 'This code already exists'
                saved = False

            if form.cleaned_data['move_choice']:
                target = form.cleaned_data['target']
                position = form.cleaned_data['position']

                try:
                    area.parent=target
                    area.save()
                except InvalidMove:
                    form.errors['position'] = 'This move is invalid'
                    saved = False

            if saved:
                form = LocationForm()
                return render_to_response("simple_locations/location_edit.html", {"form":form, 'nodes':Area.tree.all()},
                                          context_instance=RequestContext(req))
            else:
                return render_to_response("simple_locations/location_edit.html",
                                          {"form":form, 'item': location, 'nodes':Area.tree.all()},
                                          context_instance=RequestContext(req))

        else:
            return render_to_response("simple_locations/location_edit.html",
                                      { 'form': form, 'item': location },
                                      context_instance=RequestContext(req))
    else:
        default_data = {}
        default_data['pk'] = location.pk
        default_data['name'] = location.name
        default_data['code'] = location.code
        default_data['move_choice'] = False
        if location.kind:
            default_data['kind'] = location.kind.pk
        if location.parent:
            default_data['target'] = location.parent
            default_data['position'] = 'last-child'
        if location.location:
            default_data['lat'] = location.location.latitude
            default_data['lon'] = location.location.longitude
        form = LocationForm(default_data)
        return render_to_response("simple_locations/location_edit.html",
                                  {'form':form, 'nodes':Area.tree.all(),'item':location},
                                  context_instance=RequestContext(req))

def delete_location(request, area_id):
    node = get_object_or_404(Area, pk=area_id)
    if request.method == 'POST':
        node.delete()

    return HttpResponseRedirect('/simple_locations/render_tree')

@cache_control(no_cache=True)
def render_location(request):
    nodes = Area.tree.all()
    return render_to_response('simple_locations/treepanel.html',{'nodes':nodes})


def area_search(request):
    import json
    areadetails = []
    if request.GET.__contains__('query'):
        objects = Area.objects.filter(name__iregex=request.GET['query'])
    else:
        objects = Area.objects.all()
    for area in objects:
        areadetail = {}
        areadetail['value'] = area.pk
        areadetail['name']= area.name
        areadetail['kind'] = area.kind.name
        if area.parent:
            areadetail['parentname'] = area.parent.name
            areadetail['parentkind'] = area.parent.kind.name
        areadetails.append(areadetail)
    return HttpResponse(json.dumps(areadetails))



#!/usr/bin/env python
# encoding=utf-8
# vim: ai ts=4 sts=4 et sw=4


from django.contrib.gis.db.models import MultiPolygonField
from django.db import models
from django.utils.translation import ugettext as _, ugettext_lazy as __
from django.utils.encoding import python_2_unicode_compatible
# from code_generator.fields import CodeField # removed so that we can use
# South
from mptt.models import MPTTModel


@python_2_unicode_compatible
class Point(models.Model):

    class Meta:
        verbose_name = __("Point")
        verbose_name_plural = __("Points")
        app_label = 'simple_locations'

    latitude = models.DecimalField(max_digits=13, decimal_places=10)
    longitude = models.DecimalField(max_digits=13, decimal_places=10)

    def __str__(self):
        return _(u"%(lat)s, %(lon)s") % {'lat': self.latitude,
                                         'lon': self.longitude}


@python_2_unicode_compatible
class AreaType(models.Model):

    class Meta:
        verbose_name = __("Area Type")
        verbose_name_plural = __("Area Types")
        app_label = 'simple_locations'

    name = models.CharField(max_length=100)
    slug = models.CharField(max_length=30, unique=True)

    def __str__(self):
        return _(self.name)


@python_2_unicode_compatible
class Area(MPTTModel):

    class Meta:
        unique_together = ('code', 'kind')
        verbose_name = __("Area")
        verbose_name_plural = __("Areas")
        app_label = 'simple_locations'

    class MPTTMeta:
        parent_attr = 'parent'
        order_insertion_by = ['name']

    name = models.CharField(max_length=100)
    code = models.CharField(max_length=50,)  # was CodeField
    kind = models.ForeignKey('AreaType', blank=True, null=True, on_delete=models.CASCADE)
    location = models.ForeignKey(Point, blank=True, null=True, on_delete=models.CASCADE)
    geom = MultiPolygonField(srid=4326, null=True)
    parent = models.ForeignKey('self', blank=True, null=True, related_name='children', on_delete=models.CASCADE)

    def delete(self):
        super(Area, self).delete()

    def get_ancestor_at_level(self, level=2):
        """Get the area ancestor at a given level

        Will travel the tree until it reaches the level or return self if already under that level"""
        if self.get_level() <= level:
            return self
        return self.get_ancestors()[level]

    def display_name_and_type(self):
        '''Area name and type

        Example District of Bamako'''
        return u"%(type)s of %(area)s" % {'type': self.kind.name, 'area': self.name}

    def display_with_parent(self):
        '''Print Area name and kind and parent name and kind

        Example: Aldeia of Baha-Neo in Suco of Lia Ruca'''
        if not self.parent:
            return self.display_name_and_type()
        elif self.kind.name == 'District':
            return self.display_name_and_type()
        else:
            return u"%(this)s in %(parent)s" % \
                {
                    'this': self.display_name_and_type(),
                    'parent': self.parent.display_name_and_type()
                }

    def __str__(self):
        ''' print Area name from its Kind and parent

        Example: Bamako '''

        # don't add-in kind if kind name is already part of name.
        # if (not self.parent) or (not self.kind) or self.name.startswith(self.kind.name):
        #    return self.name
        # else:
        #    return _(u"%(type)s of %(area)s.") % {'type': self.kind.name, \
        #                                              'area': self.name,}

        return self.name



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Area',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=100)),
                ('code', models.CharField(max_length=50)),
                ('lft', models.PositiveIntegerField(editable=False, db_index=True)),
                ('rght', models.PositiveIntegerField(editable=False, db_index=True)),
                ('tree_id', models.PositiveIntegerField(editable=False, db_index=True)),
                ('level', models.PositiveIntegerField(editable=False, db_index=True)),
            ],
            options={
                'verbose_name': 'Area',
                'verbose_name_plural': 'Areas',
            },
        ),
        migrations.CreateModel(
            name='AreaType',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=100)),
                ('slug', models.CharField(unique=True, max_length=30)),
            ],
            options={
                'verbose_name': 'Area Type',
                'verbose_name_plural': 'Area Types',
            },
        ),
        migrations.CreateModel(
            name='Facility',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=100)),
                ('code', models.CharField(max_length=64, blank=True)),
                ('area', models.ForeignKey(related_name='facility', blank=True, to='simple_locations.Area', null=True, on_delete=models.CASCADE)),
                ('catchment_areas', models.ManyToManyField(related_name='catchment', to='simple_locations.Area')),
            ],
            options={
                'verbose_name': 'Facility',
                'verbose_name_plural': 'Facilities',
            },
        ),
        migrations.CreateModel(
            name='FacilityType',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=100)),
                ('slug', models.CharField(unique=True, max_length=30)),
            ],
            options={
                'verbose_name': 'Facility Type',
                'verbose_name_plural': 'Facility Types',
            },
        ),
        migrations.CreateModel(
            name='Point',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('latitude', models.DecimalField(max_digits=13, decimal_places=10)),
                ('longitude', models.DecimalField(max_digits=13, decimal_places=10)),
            ],
            options={
                'verbose_name': 'Point',
                'verbose_name_plural': 'Points',
            },
        ),
        migrations.AddField(
            model_name='facility',
            name='location',
            field=models.ForeignKey(blank=True, to='simple_locations.Point', null=True, on_delete=models.CASCADE),
        ),
        migrations.AddField(
            model_name='facility',
            name='parent',
            field=models.ForeignKey(related_name='facility', blank=True, to='simple_locations.Facility', null=True, on_delete=models.CASCADE),
        ),
        migrations.AddField(
            model_name='facility',
            name='type',
            field=models.ForeignKey(blank=True, to='simple_locations.FacilityType', null=True, on_delete=models.CASCADE),
        ),
        migrations.AddField(
            model_name='area',
            name='kind',
            field=models.ForeignKey(blank=True, to='simple_locations.AreaType', null=True, on_delete=models.CASCADE),
        ),
        migrations.AddField(
            model_name='area',
            name='location',
            field=models.ForeignKey(blank=True, to='simple_locations.Point', null=True, on_delete=models.CASCADE),
        ),
        migrations.AddField(
            model_name='area',
            name='parent',
            field=models.ForeignKey(related_name='children', blank=True, to='simple_locations.Area', null=True, on_delete=models.CASCADE),
        ),
        migrations.AlterUniqueTogether(
            name='area',
            unique_together=set([('code', 'kind')]),
        ),
    ]



from setuptools import setup, find_packages


setup(
    name='simple_locations',
    version='2.6.1',
    license="BSD",

    description="The common location package used for catalpa's projects.",
    long_description=open('README.md').read(),
    author='Anders Hofstee, Nicoas Hoibian',
    author_email='a.hofstee@catalpainternational.org',
    url='https://github.com/catalpainternational/simple_locations',

    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,
    install_requires=['django-mptt==0.9.0'],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Framework :: Django',
    ]
)



#!/usr/bin/env python
# encoding=utf-8
# maintainer rgaudin

from django.contrib import admin
from django import forms

from mptt.admin import MPTTModelAdmin

from simple_locations.models import Point, AreaType, Area


class PointAdmin(admin.ModelAdmin):
    list_display = ('id', 'latitude', 'longitude')


class AreaTypeAdmin(admin.ModelAdmin):
    list_display = ('slug', 'name')


class AreaAdmin(MPTTModelAdmin):
    list_display = ( 'name', 'kind', 'location', 'code')
    search_fields = ['code', 'name']
    list_filter = ('kind',)
    

admin.site.register(Point, PointAdmin)
admin.site.register(AreaType, AreaTypeAdmin)
admin.site.register(Area, AreaAdmin)
